Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 223969)
+++ channels/chan_sip.c	(working copy)
@@ -1227,6 +1227,7 @@
 static int global_rtpkeepalive;		/*!< Send RTP keepalives */
 static int global_reg_timeout;		/*!< Global time between attempts for outbound registrations */
 static int global_regattempts_max;	/*!< Registration attempts before giving up */
+static int global_shrinkcallerid;	/*!< enable or disable shrinking of caller id  */
 static int global_callcounter;		/*!< Enable call counters for all devices. This is currently enabled by setting the peer
 						call-limit to INT_MAX. When we remove the call-limit from the code, we can make it
 						with just a boolean flag in the device structure */
@@ -13427,7 +13428,7 @@
 		cid_num = (char *)p->cid_num;
 	} else if (!strncasecmp(start, "sip:", 4)) {
 		cid_num = start + 4;
-		if (ast_is_shrinkable_phonenumber(cid_num))
+		if (global_shrinkcallerid && ast_is_shrinkable_phonenumber(cid_num))
 			ast_shrink_phone_number(cid_num);
 		start = end;
 
@@ -13506,7 +13507,7 @@
 	if (strncasecmp(start, "sip:", 4))
 		return 0;
 	cid_num = start + 4;
-	if (ast_is_shrinkable_phonenumber(cid_num))
+	if (global_shrinkcallerid && ast_is_shrinkable_phonenumber(cid_num))
 		ast_shrink_phone_number(cid_num);
 	start = end;
 
@@ -14354,7 +14355,7 @@
 		if (!get_rpid(p, req)) {
 			if (!ast_strlen_zero(peer->cid_num)) {
 				char *tmp = ast_strdupa(peer->cid_num);
-				if (ast_is_shrinkable_phonenumber(tmp))
+				if (global_shrinkcallerid && ast_is_shrinkable_phonenumber(tmp))
 					ast_shrink_phone_number(tmp);
 				ast_string_field_set(p, cid_num, tmp);
 			}
@@ -14463,7 +14464,7 @@
 			<sip:8164444422;phone-context=+1@1.2.3.4:5060;user=phone;tag=SDadkoa01-gK0c3bdb43>
 		*/
 		tmp = strsep(&tmp, ";");
-		if (ast_is_shrinkable_phonenumber(tmp))
+		if (global_shrinkcallerid && ast_is_shrinkable_phonenumber(tmp))
 			ast_shrink_phone_number(tmp);
 		ast_string_field_set(p, cid_num, tmp);
 	}
@@ -25051,6 +25052,7 @@
 	global_t1min = DEFAULT_T1MIN;
 	global_qualifyfreq = DEFAULT_QUALIFYFREQ;
 	global_t38_maxdatagram = -1;
+	global_shrinkcallerid = 1;
 
 	sip_cfg.matchexterniplocally = DEFAULT_MATCHEXTERNIPLOCALLY;
 
@@ -25503,6 +25505,14 @@
 			mark_parsed_methods(&sip_cfg.disallowed_methods, disallow);
 		} else if (!strcasecmp(v->name, "constantssrc")) {
 			ast_set2_flag(&global_flags[1], ast_true(v->value), SIP_PAGE2_CONSTANT_SSRC);
+		} else if (!strcasecmp(v->name, "shrinkcallerid")) {
+			if (ast_true(v->value)) {
+				global_shrinkcallerid = 1;
+			} else if (ast_false(v->value)) {
+				global_shrinkcallerid = 0;
+			} else {
+				ast_log(LOG_WARNING, "shrinkcallerid value %s is not valid at line %d.\n", v->value, v->lineno);
+			}
 		}
 	}
 
Index: channels/chan_iax2.c
===================================================================
--- channels/chan_iax2.c	(revision 223969)
+++ channels/chan_iax2.c	(working copy)
@@ -421,7 +421,7 @@
 #define IAX_SENDCONNECTEDLINE   (uint64_t)(1 << 28)   /*!< Allow sending of connected line updates */
 #define IAX_RECVCONNECTEDLINE   (uint64_t)(1 << 29)   /*!< Allow receiving of connected line updates */
 #define IAX_FORCE_ENCRYPT       (uint64_t)(1 << 30)   /*!< Forces call encryption, if encryption not possible hangup */
-
+#define IAX_SHRINKCALLERID      (uint64_t)(1 << 31)   /*!< Turn on and off caller id shrinking */
 static int global_rtautoclear = 120;
 
 static int reload_config(void);
@@ -7224,7 +7224,9 @@
 	if (ies->called_number)
 		ast_string_field_set(iaxs[callno], exten, ies->called_number);
 	if (ies->calling_number) {
-		ast_shrink_phone_number(ies->calling_number);
+		if (ast_test_flag64(&globalflags, IAX_SHRINKCALLERID)) { 
+			ast_shrink_phone_number(ies->calling_number);
+		}
 		ast_string_field_set(iaxs[callno], cid_num, ies->calling_number);
 	}
 	if (ies->calling_name)
@@ -12563,6 +12565,7 @@
 	/* Reset Global Flags */
 	memset(&globalflags, 0, sizeof(globalflags));
 	ast_set_flag64(&globalflags, IAX_RTUPDATE);
+	ast_set_flag64((&globalflags), IAX_SHRINKCALLERID);
 
 #ifdef SO_NO_CHECK
 	nochecksums = 0;
@@ -12829,10 +12832,18 @@
 			if (sscanf(v->value, "%10hu", &global_maxcallno_nonval) != 1) {
 				ast_log(LOG_WARNING, "maxcallnumbers_nonvalidated must be set to a valid number.  %s is not valid at line %d.\n", v->value, v->lineno);
 			}
-		} else if(!strcasecmp(v->name, "calltokenoptional")) {
+		} else if (!strcasecmp(v->name, "calltokenoptional")) {
 			if (add_calltoken_ignore(v->value)) {
 				ast_log(LOG_WARNING, "Invalid calltokenoptional address range - '%s' line %d\n", v->value, v->lineno);
 			}
+		} else if (!strcasecmp(v->name, "shrinkcallerid")) {
+			if (ast_true(v->value)) {
+				ast_set_flag64((&globalflags), IAX_SHRINKCALLERID);
+			} else if (ast_false(v->value)) {
+				ast_clear_flag64((&globalflags), IAX_SHRINKCALLERID);
+			} else {
+				ast_log(LOG_WARNING, "shrinkcallerid value %s is not valid at line %d.\n", v->value, v->lineno);
+			}
 		}/*else if (strcasecmp(v->name,"type")) */
 		/*	ast_log(LOG_WARNING, "Ignoring %s\n", v->name); */
 		v = v->next;
